#ifndef RL_Animation_H
#define RL_Animation_H 1

/***************************************************************************/

#include "main.h"
#include "rl_time.h"

/***************************************************************************/

/*
** All available animation modes 
**
** NOTE: If you insert, remove or swap entries here, you need
** to adapt the matching TAB_CYLE entry in rl_wifi.cpp, or the
** web interface wont match the modes anymore.
*/

enum{
  ANIMATION_STATIC = 0,           /* show a fixed color */
  ANIMATION_RANDOM,               /* cycle randomly though our favorites */
  ANIMATION_RAINBOW,              /* rotating Rainbow */
  ANIMATION_RAINBOWSPARKLE,       /* rotating Rainbow with sparkles */
  ANIMATION_FIREPLACE,            /* fire place style */
  ANIMATION_ICEPLACE,             /* ice style fire */
  ANIMATION_GRASSPLACE,           /* grass style fire */
  ANIMATION_RGBFADE,              /* fade between Red, Green, Blue */
  ANIMATION_TWINKLE,              /* change random dot with random RGB */
  ANIMATION_RACE,                 /* Trailing pixels racing "randomly" */
  ANIMATION_COLORCOMET,           /* flying comet */
  ANIMATION_COLORCOMETSTATIC,     /* flying comet single color */
  ANIMATION_LAST                  /* no real mode, keep at the end */
};

#define ANIMATION_RANDOM_MODEFLAGS     ( 1 << ANIMATION_RAINBOW ) | ( 1 << ANIMATION_FIREPLACE ) | ( 1 << ANIMATION_RGBFADE ) | ( 1 << ANIMATION_TWINKLE ) | ( 1 << ANIMATION_RACE ) | ( 1 << ANIMATION_COLORCOMET )

/***************************************************************************/

#define ANIMATION_FADESTEP          16  /* x/256 color steps */
#define ANIMATION_FADEDELAY         50  /* delay in FPS */

#define ANIMATION_BRIGHTNESSUP     180  /* for animation effects using a brigher base color */
#define ANIMATION_BRIGHTNESSDOWN    20  /* for animation effects using a darker base color */

/* twinkle configuration */

#define TWINKLE_FADEAWAY 4

/* fire place, ice fire place configuration */

#define FIRE_SPEED                   2
#define FIRE_DOWN                   55
#define FIRE_UP                    120
#define FIRE_HEIGHT                ( ( RINGLIGHT_SEGMENT_LEDS / 2 ) + 1 ) /* we have a mirrored ring, so only burn to the top middle */
#define FIRE_SPARK_RADIUS            2  /* this the bottom radius, as we mirrow the fire in x direction */
#define FIRE_SPARK_MININTENSITY    160

/* rainbow sparkle */

#define RAINBOW_SPARKLERATE         25  /* percentage change of having a sparkle */

/* race configuration */

#define RACE_NUMBEROF                4
#define RACE_TRAIL_FADEAWAY         27

/* color comet (static) configuration */

#define COLORCOMET_BPM                    14
#define COLORCOMET_TRAIL_FADEAWAY         27

/***************************************************************************/

/*
** RL_Animation
**
** keeps constant and running animation data
*/


struct RL_Animation {
	uint16_t Flags;                 /* is this animation active? */
	uint16_t Mode;                  /* buffered rlc.rlc_AnimationMode */
	uint16_t RandomMode;            /* buffered rlc.rlc_AnimationMode (random) */
	 int16_t FadeBrightness;        /* fading when changing animation mode (0-100%) */
	uint16_t FadeDelay;             /* blank time between fading */
	uint16_t Phase;                 /* current step of the animation, usage depends on type of animation */
	uint16_t InternalCount1;        /* depends on type of animation */
	uint16_t InternalCount2;        /* depends on type of animation */
};

/*
** some flags for animation fading control
*/

#define ACF_ACTIVE              ( 1 << 0 ) /* running */
#define ACF_FADEIN              ( 1 << 1 ) /* fade in state */
#define ACF_FADEOUT             ( 1 << 2 ) /* fade out state */
#define ACF_FADEDELAY           ( 1 << 3 ) /* black pause state */

#define ACF_MODEMASK            (              ACF_FADEIN | ACF_FADEOUT | ACF_FADEDELAY )

extern struct RL_Animation animation;

/***************************************************************************/


/*
** prototypes
*/

void Animation_Setup( void );
void Animation_Randomizer( void );
void Animation_Animate( void );

/***************************************************************************/

#endif /* RL_Animation_H */
