#ifndef RL_HTML_H
#define RL_HTML_H 1

/***************************************************************************/

#include "main.h"

/***************************************************************************/

/*
** struct to webpage defines and helpers 
*/

struct ws_Settings {
	int16_t     ws_PageID;
	int16_t     ws_Type;
	int32_t     ws_Flags;
	const char *ws_ID;
	const char *ws_Name;
	uint32_t   *ws_Ptr;     /* data pointer (color, flags, ... ) */
	uint32_t    ws_Value1;  /* value1 depends on type */
	uint32_t    ws_Value2;  /* value1 depends on type */
	uint32_t    ws_Value3;  /* value1 depends on type */
	const char *ws_Action;
};

/***************************************************************************/

/*
** web structure control flags
*/

#define WSF_ST          2   /* same tab */
#define WSF_SR          4   /* same row */
#define WSF_C           8   /* center */
#define WSF_L          16   /* left */
#define WSF_R          32   /* right */
#define WSF_D          64   /* disabled */
#define WSF_H         128   /* hidden */

#define WSF_SRT (WSF_ST|WSF_SR)

/*
** helper for all entries 
*/

#define WS_PAGEID                    tab_uidata[ i ].ws_PageID
#define WS_TYPE                      tab_uidata[ i ].ws_Type
#define WS_FLAGS                     tab_uidata[ i ].ws_Flags
#define WS_ID                        tab_uidata[ i ].ws_ID
#define WS_NAME                      tab_uidata[ i ].ws_Name
#define WS_ACTION                    tab_uidata[ i ].ws_Action

/* helper for color object */

#define WS_COLORRGB                  tab_uidata[ i ].ws_Ptr
#define WS_COLORHSV                  tab_uidata[ i ].ws_Value

/* helper for label object */

#define WS_LABELFOR  ((char*)        tab_uidata[i].ws_Ptr )

/* helper for slider object */

#define WS_SLIDERPTR                 tab_uidata[ i ].ws_Ptr
#define WS_SLIDERMIN                 tab_uidata[ i ].ws_Value1
#define WS_SLIDERMAX                 tab_uidata[ i ].ws_Value2
#define WS_SLIDERSTEP                tab_uidata[ i ].ws_Value3

/* helper for number object */

#define WS_NUMBERPTR                 tab_uidata[ i ].ws_Ptr
#define WS_NUMBERMIN                 tab_uidata[ i ].ws_Value1
#define WS_NUMBERMAX                 tab_uidata[ i ].ws_Value2

/* helper for helper for flag object */

#define WS_FLAGPTR                   tab_uidata[ i ].ws_Ptr
#define WS_FLAGMASK                  tab_uidata[ i ].ws_Value1

/* helper for helper for cycle object */

#define WS_CYCLEACTIVE               tab_uidata[ i ].ws_Ptr
#define WS_CYCLEARRAY    ((char **) (tab_uidata[ i ].ws_Value1))

/* helper for helper for yes/no/on/off object */

#define WS_ONOFFPTR                  tab_uidata[ i ].ws_Ptr
#define WS_ONOFFMASK                 tab_uidata[ i ].ws_Value1

/* helper for button object */

#define WS_BUTTONCOMMAND             tab_uidata[ i ].ws_Value1

/* helper for text object */

#define WS_TEXTPTR                   tab_uidata[ i ].ws_Ptr
#define WS_TEXTMAXLEN                tab_uidata[ i ].ws_Value1
#define WS_TEXT           ((char *) (tab_uidata[ i ].ws_Ptr))
#define WS_TEXTONCHANGE   ((char *) (tab_uidata[ i ].ws_Value2))

/* helper for refflag object */

#define WS_REFFLAGPTR                tab_uidata[ i ].ws_Ptr
#define WS_REFFLAGREF *((uint32_t*) (tab_uidata[ i ].ws_Value1))

/***************************************************************************/

/*
** web structure type values
*/

#define WSTYPE_INVALID    0  /* end of table */
#define WSTYPE_SPACER     1  /* vertical spacing */
#define WSTYPE_TITLE      2  /* just some title text */
#define WSTYPE_LABEL      3  /* just some text */
#define WSTYPE_HSPACE     4  /* an empty cell */
#define WSTYPE_COLOR      5  /* color picker */
#define WSTYPE_YESNO      6  /* yes/no cycle element */
#define WSTYPE_ONOFF      7  /* on/off cycle element */
#define WSTYPE_SLIDER     8  /* numeric element */
#define WSTYPE_CYCLE      9  /* cycle element */
#define WSTYPE_NUMBER    10  /* numeric element */
#define WSTYPE_CHECKBOX  11  /* box with checkmark */
#define WSTYPE_BUTTON    12  /* action button */
#define WSTYPE_TEXT      13  /* text input field */
#define WSTYPE_PASSWORD  14  /* password input field */
#define WSTYPE_REFFLAG   15  /* reference based flag control  (1<<ref) */
#define WSTYPE_PLAIN     16  /* plain output */

/***************************************************************************/

/*
** prototypes
*/

bool HTML_CreateUIFromTable( uint32_t pageid, struct ws_Settings *tab_uidata );

/***************************************************************************/

#endif /* RL_HTML_H */