/*
**   rl_ota.cpp
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#include "main.h"
#include "rl_ota.h"
#include "rl_wifi.h" /* password from credentials */

/***************************************************************************/

volatile uint16_t ota_updatestate;         /* 0 -> no ota update right now / != 0 -> ota update in progress */
volatile uint16_t ota_updateprogress;      /* contains 0-100% or other values depending on state */

/***************************************************************************/

/* /// OTA_Startup()
**
*/

/***************************************************************************/

void OTA_Startup( void )
{
	debug(( "\nOTA_Startup()\n" ));

	ota_updatestate    = OTA_UPDATESTATE_IDLE;     /* no ota running */
	ota_updateprogress = 0;                        /* contains 0-100% */

/* /// ArduinoOTA.onStart()
**
*/

/***************************************************************************/

ArduinoOTA.onStart( [] ()
{
	ota_updateprogress = OTA_PROGRESS_FILESYSTEM;

	if( ArduinoOTA.getCommand() == U_FLASH ) {
		ota_updateprogress = OTA_PROGRESS_UFLASH;
	}
	ota_updatestate = OTA_UPDATESTATE_STARTING;

	debug(( "OTA_Update type: %s\n", ( ota_updateprogress == OTA_PROGRESS_FILESYSTEM ) ? "filesystem" : "sketch" ));
});

/* \\\ ArduinoOTA.onStart */

/* /// ArduinoOTA.onEnd()
**
*/

/***************************************************************************/

ArduinoOTA.onEnd( [] ()
{
	debug(( "\nOTA_End()\n" ));

	ota_updatestate = OTA_UPDATESTATE_IDLE; /* reset OTA update state */
	delay(1000);  /* give main task time to clear the display */
});
/* \\\ ArduinoOTA.onEnd() */

/* /// ArduinoOTA.onProgress()
**
*/

/***************************************************************************/

ArduinoOTA.onProgress( [] ( unsigned int progress, unsigned int total )
{
	ota_updateprogress = ( progress / ( total / 100 ) );
});
/* \\\ ArduinoOTA.onProgress */

/* /// ArduinoOTA.onError()
**
*/

/***************************************************************************/

ArduinoOTA.onError( [] (ota_error_t error ) {
#if ENABLE_DEBUG
	debug(( "OTA_Error[%u]: ", error ));
		 if ( error == OTA_AUTH_ERROR    ) debug(( "Auth Failed\n" ));
	else if ( error == OTA_BEGIN_ERROR   ) debug(( "Begin Failed\n" ));
	else if ( error == OTA_CONNECT_ERROR ) debug(( "Connect Failed\n" ));
	else if ( error == OTA_RECEIVE_ERROR ) debug(( "Receive Failed\n" ));
	else if ( error == OTA_END_ERROR     ) debug(( "End Failed\n" ));
#endif
	ota_updatestate    = OTA_UPDATESTATE_ERROR;
	ota_updateprogress = error;
});
/* \\\ ArduinoOTA.onError() */


	ArduinoOTA.setHostname( APPLICATIONNAME_NS );
	ArduinoOTA.setPassword( OTA_PASSWORD );

	ota_updatestate = OTA_UPDATESTATE_IDLE; /* reset OTA update state */

	ArduinoOTA.begin();

	debug(( "\nOTA initialized\nHostname is: '%s'\nIP address: %s\n", APPLICATIONNAME_NS, WiFi.localIP().toString().c_str() ));
}
/* \\\ OTA_Startup() */