#ifndef RL_RENDER_H
#define RL_RENDER_H 1

/***************************************************************************/

#include "main.h"
#include "rl_display.h"

/***************************************************************************/

/*
** Now this gets complicated. We cannot simply multiply width and height to 
** get the total size of the led field, because the top line is also part of 
** height, as the corners of the 7 segments are not containing LEDs.
**
** As you can see here, this small example is not 12x4 LEDs. It is 15x7.
**
**            .------.------.
**            |      |      |
**            |      |      |
**            .------.------.
**            |      |      |
**            |      |      |
**            .------.------.
**
** Shifting them together would make the stuff even more complicated, so we
** define the entire render matrix bigger than it actually is.
**
** This way coordinate driven LED control works as we have a conform and 
** even grid.
**
*/

#define RENDER_LED_INVALID           32767

/***************************************************************************/

extern CRGB *renderbuffer;

/***************************************************************************/

/*
** Enhanced HSV structure
**
** This structure is used to change, modify or 
** copy brightness information of CRGB colors.
**
** By default this is not possible using CHSV as 
** values get cut off. This structure allows more or
** less lossless color operation.
*/

struct EHSV {
	float H;
	float S;
	float V;
};

/***************************************************************************/

/*
** prototypes
*/

void Render_Setup( void );

//void Render_DotColor( uint32_t x, uint32_t y, CRGB color );
#define Render_DotColor( tx, ty, tcolor )  renderbuffer[ (tx) + (ty) * RENDER_BUFFER_WIDTH ] = tcolor

CRGB      Render_ChangeBrightness( CRGB color, uint32_t percent );

CRGB      Render_FadeToColor( CRGB *current, const CRGB *destination, uint8_t difference );

void      Render_ClearBuffer( void );
void      Render_SetBuffer( CRGB color );

/***************************************************************************/

#endif /* RL_RENDER_H */

